(function() {
  var assert, browserChannel, buffer, connect, createServer, expect, expectCalls, http, ieJunk, makePassPart, parse, querystring, readLengthPrefixedJSON, testCase, timer;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  testCase = require('nodeunit').testCase;
  connect = require('connect');
  browserChannel = require('./lib').server;
  http = require('http');
  parse = require('url').parse;
  assert = require('assert');
  querystring = require('querystring');
  timer = require('timerstub');
  browserChannel._setTimerMethods(timer);
  expectCalls = function(n, callback) {
    var remaining;
    if (n === 0) {
      return callback();
    }
    remaining = n;
    return function() {
      remaining--;
      if (remaining === 0) {
        return callback();
      } else if (remaining < 0) {
        throw new Error("expectCalls called more than " + n + " times");
      }
    };
  };
  makePassPart = function(test, n) {
    return expectCalls(n, function() {
      return test.done();
    });
  };
  buffer = function(res, callback) {
    var data;
    data = [];
    res.on('data', function(chunk) {
      return data.push(chunk.toString('utf8'));
    });
    return res.on('end', function() {
      return callback(data.join(''));
    });
  };
  expect = function(res, str, callback) {
    var data, endlistener, listener;
    data = '';
    res.on('end', endlistener = function() {
      console.warn('Connection ended prematurely');
      return assert.strictEqual(data, str);
    });
    return res.on('data', listener = function(chunk) {
      data += chunk.toString('utf8');
      if (data.length >= str.length) {
        assert.strictEqual(data, str);
        res.removeListener('data', listener);
        res.removeListener('end', endlistener);
        return callback();
      }
    });
  };
  readLengthPrefixedJSON = function(res, callback) {
    var data, length, listener;
    data = '';
    length = null;
    return res.on('data', listener = function(chunk) {
      var lines, obj;
      data += chunk.toString('utf8');
      if (length === null) {
        lines = data.split('\n');
        if (lines.length > 1) {
          length = parseInt(lines.shift());
          data = lines.join('\n');
        }
      }
      if (data.length === length) {
        obj = JSON.parse(data);
        res.removeListener('data', listener);
        return callback(obj);
      } else if (data.length > length) {
        throw new Error("Read more bytes from stream than expected");
      }
    });
  };
  ieJunk = "7cca69475363026330a0d99468e88d23ce95e222591126443015f5f462d9a177186c8701fb45a6ffee0daf1a178fc0f58cd309308fba7e6f011ac38c9cdd4580760f1d4560a84d5ca0355ecbbed2ab715a3350fe0c479050640bd0e77acec90c58c4d3dd0f5cf8d4510e68c8b12e087bd88cad349aafd2ab16b07b0b1b8276091217a44a9fe92fedacffff48092ee693af\n";
  createServer = function(opts, method, callback) {
    var bc, server;
    if (typeof opts === 'function') {
      callback = method;
      method = opts;
      bc = browserChannel(method);
    } else {
      bc = browserChannel(opts, method);
    }
    server = connect(bc, function(req, res, next) {
      res.writeHead(200, 'OK', {
        'Content-Type': 'text/plain'
      });
      return res.end('Other middleware');
    });
    return server.listen(function() {
      var port;
      port = server.address().port;
      return callback(server, port);
    });
  };
  module.exports = testCase({
    setUp: function(callback) {
      this.onClient = function(client) {};
      return createServer((__bind(function(client) {
        return this.onClient(client);
      }, this)), __bind(function(server, port) {
        this.server = server;
        this.port = port;
        this.get = __bind(function(path, callback) {
          return http.get({
            host: 'localhost',
            path: path,
            port: this.port
          }, callback);
        }, this);
        this.post = __bind(function(path, data, callback) {
          var req;
          req = http.request({
            method: 'POST',
            host: 'localhost',
            path: path,
            port: this.port
          }, callback);
          return req.end(data);
        }, this);
        return callback();
      }, this));
    },
    tearDown: function(callback) {
      this.server.on('close', callback);
      return this.server.close();
    },
    'GET /test/?mode=INIT with no baseprefix set returns [null, null]': function(test) {
      return this.get('/channel/test?VER=8&MODE=init', function(response) {
        test.strictEqual(response.statusCode, 200);
        return buffer(response, function(data) {
          test.strictEqual(data, '[null,null]');
          return test.done();
        });
      });
    },
    'GET /test/?mode=INIT with a basePrefix set returns [basePrefix, null]': function(test) {
      return createServer({
        hostPrefixes: ['chan']
      }, (function() {}), function(server, port) {
        return http.get({
          path: '/channel/test?VER=8&MODE=init',
          host: 'localhost',
          port: port
        }, function(response) {
          test.strictEqual(response.statusCode, 200);
          return buffer(response, function(data) {
            test.strictEqual(data, '["chan",null]');
            server.close();
            return test.done();
          });
        });
      });
    },
    'The test channel responds at a bound custom endpoint': function(test) {
      return createServer({
        base: '/foozit'
      }, (function() {}), function(server, port) {
        return http.get({
          path: '/foozit/test?VER=8&MODE=init',
          host: 'localhost',
          port: port
        }, function(response) {
          test.strictEqual(response.statusCode, 200);
          return buffer(response, function(data) {
            test.strictEqual(data, '[null,null]');
            server.close();
            return test.done();
          });
        });
      });
    },
    'binding the server to a custom url without a leading slash works': function(test) {
      return createServer({
        base: 'foozit'
      }, (function() {}), function(server, port) {
        return http.get({
          path: '/foozit/test?VER=8&MODE=init',
          host: 'localhost',
          port: port
        }, function(response) {
          test.strictEqual(response.statusCode, 200);
          return buffer(response, function(data) {
            test.strictEqual(data, '[null,null]');
            server.close();
            return test.done();
          });
        });
      });
    },
    'binding the server to a custom url with a trailing slash works': function(test) {
      return createServer({
        base: 'foozit/'
      }, (function() {}), function(server, port) {
        return http.get({
          path: '/foozit/test?VER=8&MODE=init',
          host: 'localhost',
          port: port
        }, function(response) {
          test.strictEqual(response.statusCode, 200);
          return buffer(response, function(data) {
            test.strictEqual(data, '[null,null]');
            server.close();
            return test.done();
          });
        });
      });
    },
    'getting a url outside of the bound range gets passed to other middleware': function(test) {
      return this.get('/otherapp', function(response) {
        test.strictEqual(response.statusCode, 200);
        return buffer(response, function(data) {
          test.strictEqual(data, 'Other middleware');
          return test.done();
        });
      });
    },
    'getting a wacky url inside the bound range returns 404': function(test) {
      return this.get('/channel/doesnotexist', function(response) {
        test.strictEqual(response.statusCode, 404);
        return test.done();
      });
    },
    'Getting test phase 2 returns 11111 then 2': (function() {
      var makeTest;
      makeTest = function(type, message1, message2) {
        return function(test) {
          return this.get("/channel/test?VER=8&TYPE=" + type, function(response) {
            test.strictEqual(response.statusCode, 200);
            return expect(response, message1, function() {
              var f;
              response.on('data', f = function() {
                throw new Error('should not get more data so early');
              });
              return timer.wait(1999, function() {
                return setTimeout(function() {
                  response.removeListener('data', f);
                  return timer.wait(1, function() {
                    return expect(response, message2, function() {
                      return response.once('end', function() {
                        return test.done();
                      });
                    });
                  });
                }, 50);
              });
            });
          });
        };
      };
      return {
        'xmlhttp': makeTest('xmlhttp', '11111', '2'),
        'html': makeTest('html', "<html><body><script>try {parent.m(\"11111\")} catch(e) {}</script>\n" + ieJunk, '<script>try {parent.m("2")} catch(e) {}</script>\n<script>try  {parent.d(); }catch (e){}</script>\n'),
        'html with a host prefix': makeTest('html&DOMAIN=foo.bar.com', "<html><body><script>try{document.domain=\"foo.bar.com\";}catch(e){}</script>\n<script>try {parent.m(\"11111\")} catch(e) {}</script>\n" + ieJunk, '<script>try {parent.m("2")} catch(e) {}</script>\n<script>try  {parent.d(); }catch (e){}</script>\n')
      };
    })(),
    'Getting /test/* without VER=8 returns an error': (function() {
      var check400;
      check400 = function(path) {
        return function(test) {
          return this.get(path, function(response) {
            test.strictEqual(response.statusCode, 400);
            return test.done();
          });
        };
      };
      return {
        'phase 1, ver 7': check400('/channel/test?VER=7&MODE=init'),
        'phase 1, no version': check400('/channel/test?MODE=init'),
        'phase 2, ver 7, xmlhttp': check400('/channel/test?VER=7&TYPE=xmlhttp'),
        'phase 2, no version, xmlhttp': check400('/channel/test?TYPE=xmlhttp')
      };
    })(),
    'A client connects if it POSTs the right connection stuff': function(test) {
      var id;
      id = null;
      this.onClient = function(client) {
        test.ok(client);
        test.strictEqual(typeof client.id, 'string');
        test.strictEqual(client.state, 'init');
        test.strictEqual(client.appVersion, '99');
        id = client.id;
        return client.on('map', function() {
          throw new Error('Should not have received data');
        });
      };
      return this.post('/channel/bind?VER=8&RID=1000&CVER=99&t=1&junk=asdfasdf', 'count=0', __bind(function(res) {
        var expected;
        expected = (JSON.stringify([[0, ['c', id, null, 8]]])) + '\n';
        return buffer(res, function(data) {
          test.strictEqual(data, "" + expected.length + "\n" + expected);
          test.expect(5);
          return test.done();
        });
      }, this));
    },
    'A client connects ok even if it doesnt specify an app version': function(test) {
      var id;
      id = null;
      this.onClient = function(client) {
        test.strictEqual(client.appVersion, null);
        id = client.id;
        return client.on('map', function() {
          throw new Error('Should not have received data');
        });
      };
      return this.post('/channel/bind?VER=8&RID=1000&t=1&junk=asdfasdf', 'count=0', __bind(function(res) {
        var expected;
        expected = (JSON.stringify([[0, ['c', id, null, 8]]])) + '\n';
        return buffer(res, function(data) {
          test.strictEqual(data, "" + expected.length + "\n" + expected);
          test.expect(2);
          return test.done();
        });
      }, this));
    },
    'The client can post messages to the server during initialization': function(test) {
      this.onClient = function(client) {
        return client.on('map', function(data) {
          test.deepEqual(data, {
            k: 'v'
          });
          return test.done();
        });
      };
      return this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=1&ofs=0&req0_k=v', __bind(function(res) {}, this));
    },
    'The client can post messages to the server during initialization': function(test) {
      this.onClient = function(client) {
        return client.on('map', function(data) {
          test.deepEqual(data, {
            "_int_^&^%#net": 'hi"there&&\nsam'
          });
          return test.done();
        });
      };
      return this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=1&ofs=0&req0__int_%5E%26%5E%25%23net=hi%22there%26%26%0Asam', function() {});
    },
    'The client can post messages to the server after initialization': function(test) {
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      return this.onClient = __bind(function(client) {
        client.on('map', function(data) {
          test.deepEqual(data, {
            k: 'v'
          });
          return test.done();
        });
        return this.post("/channel/bind?VER=8&RID=1001&SID=" + client.id + "&AID=0", 'count=1&ofs=0&req0_k=v', __bind(function(res) {}, this));
      }, this);
    },
    'The server acknowledges forward channel messages correctly': function(test) {
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      return this.onClient = __bind(function(client) {
        return this.post("/channel/bind?VER=8&RID=1001&SID=" + client.id + "&AID=0", 'count=1&ofs=0&req0_k=v', __bind(function(res) {
          return readLengthPrefixedJSON(res, __bind(function(data) {
            test.deepEqual(data, [0, 0, 0]);
            return test.done();
          }, this));
        }, this));
      }, this);
    },
    'The server tells the client if the backchannel is alive': function(test) {
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      return this.onClient = __bind(function(client) {
        var req;
        req = this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=0&TYPE=xmlhttp", __bind(function(res) {
          return res.on('data', function() {
            throw new Error('Should not get data through backchannel');
          });
        }, this));
        return setTimeout(__bind(function() {
          return this.post("/channel/bind?VER=8&RID=1001&SID=" + client.id + "&AID=0", 'count=1&ofs=0&req0_k=v', __bind(function(res) {
            return readLengthPrefixedJSON(res, __bind(function(data) {
              test.deepEqual(data, [1, 0, 0]);
              req.abort();
              return test.done();
            }, this));
          }, this));
        }, this), 50);
      }, this);
    },
    'The server returns data on the initial connection when send is called immediately': function(test) {
      var testData;
      testData = ['hello', 'there', null, 1000, {}, [], [555]];
      this.onClient = __bind(function(client) {
        this.client = client;
        return this.client.send(testData);
      }, this);
      return this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {
        return readLengthPrefixedJSON(res, __bind(function(data) {
          test.deepEqual(data, [[0, ['c', this.client.id, null, 8]], [1, testData]]);
          return test.done();
        }, this));
      }, this));
    },
    'The server buffers data if no backchannel is available': function(test) {
      var testData;
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      testData = ['hello', 'there', null, 1000, {}, [], [555]];
      return this.onClient = __bind(function(client) {
        this.client = client;
        return process.nextTick(__bind(function() {
          var req;
          this.client.send(testData);
          return req = this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=0&TYPE=xmlhttp&CI=0", __bind(function(res) {
            return readLengthPrefixedJSON(res, __bind(function(data) {
              test.deepEqual(data, [[1, testData]]);
              req.abort();
              return test.done();
            }, this));
          }, this));
        }, this));
      }, this);
    },
    'The server returns data through the available backchannel when send is called later': function(test) {
      var testData;
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      testData = ['hello', 'there', null, 1000, {}, [], [555]];
      return this.onClient = __bind(function(client) {
        var req;
        this.client = client;
        req = this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=0&TYPE=xmlhttp&CI=0", __bind(function(res) {
          return readLengthPrefixedJSON(res, __bind(function(data) {
            test.deepEqual(data, [[1, testData]]);
            req.abort();
            return test.done();
          }, this));
        }, this));
        return setTimeout((__bind(function() {
          return this.client.send(testData);
        }, this)), 50);
      }, this);
    },
    'The backchannel is closed after each packet if chunking is turned off': function(test) {
      var testData;
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      testData = ['hello', 'there', null, 1000, {}, [], [555]];
      return this.onClient = __bind(function(client) {
        this.client = client;
        return process.nextTick(__bind(function() {
          this.client.send(testData);
          return this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=0&TYPE=xmlhttp&CI=1", __bind(function(res) {
            readLengthPrefixedJSON(res, __bind(function(data) {
              return test.deepEqual(data, [[1, testData]]);
            }, this));
            return res.on('end', function() {
              return test.done();
            });
          }, this));
        }, this));
      }, this);
    },
    'The backchannel is left open if CI=0': function(test) {
      var testData;
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      testData = ['hello', 'there', null, 1000, {}, [], [555]];
      return this.onClient = __bind(function(client) {
        this.client = client;
        return process.nextTick(__bind(function() {
          var req;
          this.client.send(testData);
          return req = this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=0&TYPE=xmlhttp&CI=0", __bind(function(res) {
            readLengthPrefixedJSON(res, __bind(function(data) {
              return test.deepEqual(data, [[1, testData]]);
            }, this));
            res.on('end', function() {
              throw new Error('connection should have stayed open');
            });
            return setTimeout(function() {
              req.abort();
              return test.done();
            }, 50);
          }, this));
        }, this));
      }, this);
    },
    'The server gives the client correctly formatted backchannel data if TYPE=html': function(test) {
      var testData;
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      testData = ['hello', 'there', null, 1000, {}, [], [555]];
      return this.onClient = __bind(function(client) {
        this.client = client;
        return process.nextTick(__bind(function() {
          this.client.send(testData);
          return this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=0&TYPE=html&CI=1", __bind(function(res) {
            return expect(res, "<html><body><script>try {parent.m(" + (JSON.stringify(JSON.stringify([[1, testData]]) + '\n')) + ")} catch(e) {}</script>\n" + ieJunk + "<script>try  {parent.d(); }catch (e){}</script>\n", __bind(function() {
              var data2, req;
              data2 = {
                other: 'data'
              };
              this.client.send(data2);
              return req = this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=1&TYPE=html&CI=0", __bind(function(res) {
                return expect(res, "<html><body><script>try {parent.m(" + (JSON.stringify(JSON.stringify([[2, data2]]) + '\n')) + ")} catch(e) {}</script>\n" + ieJunk, __bind(function() {
                  req.abort();
                  return test.done();
                }, this));
              }, this));
            }, this));
          }, this));
        }, this));
      }, this);
    },
    'The server sets the domain if we have a domain set': function(test) {
      var testData;
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      testData = ['hello', 'there', null, 1000, {}, [], [555]];
      return this.onClient = __bind(function(client) {
        this.client = client;
        return process.nextTick(__bind(function() {
          this.client.send(testData);
          return this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=0&TYPE=html&CI=1&DOMAIN=foo.com", __bind(function(res) {
            return expect(res, "<html><body><script>try{document.domain=\"foo.com\";}catch(e){}</script>\n<script>try {parent.m(" + (JSON.stringify(JSON.stringify([[1, testData]]) + '\n')) + ")} catch(e) {}</script>\n" + ieJunk + "<script>try  {parent.d(); }catch (e){}</script>\n", __bind(function() {
              var data2, req;
              data2 = {
                other: 'data'
              };
              this.client.send(data2);
              return req = this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=1&TYPE=html&CI=0&DOMAIN=foo.com", __bind(function(res) {
                return expect(res, "<html><body><script>try{document.domain=\"foo.com\";}catch(e){}</script>\n<script>try {parent.m(" + (JSON.stringify(JSON.stringify([[2, data2]]) + '\n')) + ")} catch(e) {}</script>\n" + ieJunk, __bind(function() {
                  req.abort();
                  return test.done();
                }, this));
              }, this));
            }, this));
          }, this));
        }, this));
      }, this);
    },
    'The server closes old backchannel connections': function(test) {
      var testData;
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      testData = ['hello', 'there', null, 1000, {}, [], [555]];
      return this.onClient = __bind(function(client) {
        this.client = client;
        return process.nextTick(__bind(function() {
          this.client.send(testData);
          return this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=0&TYPE=xmlhttp&CI=0", __bind(function(res) {
            return readLengthPrefixedJSON(res, __bind(function(data) {
              var req2;
              req2 = this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=1&TYPE=xmlhttp&CI=0", __bind(function(res2) {}, this));
              return res.on('end', function() {
                req2.abort();
                return test.done();
              });
            }, this));
          }, this));
        }, this));
      }, this);
    },
    'The server orders forwardchannel messages correctly using RIDs': function(test) {
      var lastVal;
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      lastVal = 0;
      return this.onClient = __bind(function(client) {
        client.on('map', function(map) {
          test.strictEqual(map.v, "" + (lastVal++), 'messages arent reordered in the server');
          if (map.v === '2') {
            return test.done();
          }
        });
        this.post("/channel/bind?VER=8&RID=1002&SID=" + client.id + "&AID=0", 'count=1&ofs=2&req0_v=2', __bind(function(res) {}, this));
        return setTimeout(__bind(function() {
          return this.post("/channel/bind?VER=8&RID=1001&SID=" + client.id + "&AID=0", 'count=2&ofs=0&req0_v=0&req1_v=1', __bind(function(res) {}, this));
        }, this), 50);
      }, this);
    },
    'Repeated forward channel messages are discarded': function(test) {
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      return this.onClient = __bind(function(client) {
        var gotMessage;
        gotMessage = false;
        client.on('map', function(map) {
          if (gotMessage === false) {
            return gotMessage = true;
          } else {
            throw new Error('got map twice');
          }
        });
        this.post("/channel/bind?VER=8&RID=1001&SID=" + client.id + "&AID=0", 'count=1&ofs=0&req0_v=0', __bind(function(res) {}, this));
        this.post("/channel/bind?VER=8&RID=1001&SID=" + client.id + "&AID=0", 'count=1&ofs=0&req0_v=0', __bind(function(res) {}, this));
        return setTimeout(function() {
          test.strictEqual(gotMessage, true);
          return test.done();
        }, 50);
      }, this);
    },
    'The server resends lost arrays if the client asks for them': function(test) {
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      return this.onClient = __bind(function(client) {
        return process.nextTick(__bind(function() {
          client.send([1, 2, 3]);
          client.send([4, 5, 6]);
          return this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=0&TYPE=xmlhttp&CI=0", __bind(function(res) {
            return readLengthPrefixedJSON(res, __bind(function(data) {
              var req;
              test.deepEqual(data, [[1, [1, 2, 3]], [2, [4, 5, 6]]]);
              return req = this.get("/channel/bind?VER=8&RID=rpc&SID=" + client.id + "&AID=1&TYPE=xmlhttp&CI=0", __bind(function(res) {
                return readLengthPrefixedJSON(res, __bind(function(data) {
                  test.deepEqual(data, [[2, [4, 5, 6]]]);
                  req.abort();
                  return test.done();
                }, this));
              }, this));
            }, this));
          }, this));
        }, this));
      }, this);
    },
    'If a client disconnects then reconnects, specifying OSID= and OAID=, the old session is destroyed': function(test) {
      this.post('/channel/bind?VER=8&RID=1000&t=1', 'count=0', __bind(function(res) {}, this));
      return this.onClient = __bind(function(client1) {
        var c1Closed;
        this.post("/channel/bind?VER=8&RID=2000&t=1&OSID=" + client1.id + "&OAID=0", 'count=0', __bind(function(res) {}, this));
        c1Closed = false;
        client.on('close', function() {
          return c1Closed = true;
        });
        return this.onClient = function(client2) {
          test.ok(c1Closed);
          test.strictEqual(client1.state, 'closed');
          return test.done();
        };
      }, this);
    }
  });
  /*
  	'The server ignores OSID and OAID if the session doesnt exist': (test) -> test.done()
  
  	'If a client sends an invalid SID in a request, the server responds with 400 Unknown SID': (test) -> test.done()
  
  	'The server uses OAID to confirm arrays in the old session before closing it': (test) -> test.done()
  
  	'The server calls send callbacks when the client confirms it has them': (test) -> test.done()
  
  	'The server calls send callbacks with an error when the client times out': (test) -> test.done()
  	'The server calls send callbacks with an error when the client is evicted': (test) -> test.done()
  	'The server calls send callbacks with an error when the client is aborted by the server': (test) -> test.done()
  
  	'Calling close() sends the stop command to the client': (test) -> test.done()
  
  	'After close() is called, no maps are emitted by the client': (test) -> test.done()
  
  	'Calling close() during the initial connection stops the client immediately': (test) -> test.done()
  
  	'Client.close() makes subsequent client messages return an error': (test) -> test.done()
  
  	'Client.abort() closes the connection': (test) -> test.done()
  
  	'The client times out after awhile': (test) -> test.done()
  
  	'The server sends heartbeat messages on the backchannel, which keeps it open': (test) -> test.done()
  
  	'If a client times out, unacknowledged messages have an error callback called': (test) -> test.done()
  
  	'The server sends accept:JSON header': (test) -> test.done()
  
  	'The server accepts JSON data': (test) -> test.done()
  	
  	'Connecting with a version thats not 8 breaks': (test) -> test.done()
  
  #server = connect browserChannel (client) ->
  #	if client.address != '127.0.0.1' or client.appVersion != '10'
  #		client.stop()
  #
  #	client.on 'map', (data) ->
  #		console.log data
  #	
  #	client.send ['hi']
  #
  #	setInterval (-> client.send ['yo dawg']), 3000
  #
  #	client.on 'reconnected', (oldSessionId) ->
  #	
  #	client.on 'destroyed', ->
  		# Clean up
  #server.listen(4321)
  
  # # Tests
  #
  # The browserchannel service exposes 2 API endpoints
  #
  # ## Test Service
  #*/
}).call(this);
